@@ -6,6 +6,7 @@ import unittest
 import numpy as np
 
 from AnyQt.QtCore import Qt, QModelIndex
+from AnyQt.QtTest import QSignalSpy
 
 from orangewidget.utils.itemmodels import \
     AbstractSortTableModel, PyTableModel, PyListModel, \
@@ -63,8 +64,8 @@ class TestPyTableModel(unittest.TestCase):
 
     def test_editable(self):
         editable_model = PyTableModel([[0]], editable=True)
-        self.assertFalse(int(self.model.flags(self.model.index(0, 0)) & Qt.ItemIsEditable))
-        self.assertTrue(int(editable_model.flags(editable_model.index(0, 0)) & Qt.ItemIsEditable))
+        self.assertFalse(self.model.flags(self.model.index(0, 0)) & Qt.ItemIsEditable)
+        self.assertTrue(editable_model.flags(editable_model.index(0, 0)) & Qt.ItemIsEditable)
 
     def test_sort(self):
         self.model.sort(1)
@@ -98,6 +99,14 @@ class TestPyTableModel(unittest.TestCase):
         self.assertEqual(self.model.rowCount(), 1)
         self.assertEqual(self.model.columnCount(), 1)
 
+    def test_init_wrap_empty(self):
+        # pylint: disable=protected-access
+        t = []
+        model = PyTableModel(t)
+        self.assertIs(model._table, t)
+        t.append([1, 2, 3])
+        self.assertEqual(list(model), [[1, 2, 3]])
+
     def test_clear(self):
         self.model.clear()
         self.assertEqual(self.model.rowCount(), 0)
@@ -131,6 +140,90 @@ class TestPyTableModel(unittest.TestCase):
                         self.model.data(self.model.index(1, 0),
                                         Qt.TextAlignmentRole))
 
+    def test_set_item_signals(self):
+        def p(*s):
+            return [[x] for x in s]
+
+        def assert_changed(startrow, stoprow, ncolumns):
+            start, stop = changed[-1][:2]
+            self.assertEqual(start.row(), startrow)
+            self.assertEqual(stop.row(), stoprow)
+            self.assertEqual(start.column(), 0)
+            self.assertEqual(stop.column(), ncolumns)
+
+        self.model.wrap(p(0, 1, 2, 3, 4, 5))
+        aboutinserted = QSignalSpy(self.model.rowsAboutToBeInserted)
+        inserted = QSignalSpy(self.model.rowsInserted)
+        aboutremoved = QSignalSpy(self.model.rowsAboutToBeRemoved)
+        removed = QSignalSpy(self.model.rowsRemoved)
+        changed = QSignalSpy(self.model.dataChanged)
+
+        # Insert rows
+        self.model[2:4] = p(6, 7, 8, 9, 10) + [[11, 2]]
+        self.assertEqual(list(self.model), p(0, 1, 6, 7, 8, 9, 10) + [[11, 2]] + p(4, 5))
+        self.assertEqual(len(changed), 1)
+        assert_changed(2, 3, 1)
+        self.assertEqual(aboutinserted[-1][1:], [4, 7])
+        self.assertEqual(inserted[-1][1:], [4, 7])
+        self.assertEqual(len(aboutremoved), 0)
+        self.assertEqual(len(removed), 0)
+
+        # Remove rows
+        self.model[2:8] = p(2, 3)
+        self.assertEqual(list(self.model), p(0, 1, 2, 3, 4, 5))
+        self.assertEqual(len(changed), 2)  # one is from before
+        assert_changed(2, 3, 0)
+        self.assertEqual(aboutremoved[-1][1:], [4, 7])
+        self.assertEqual(removed[-1][1:], [4, 7])
+        self.assertEqual(len(inserted), 1)  # from before
+        self.assertEqual(len(aboutinserted), 1)  # from before
+
+        # Change rows
+        self.model[-5:-3] = p(19, 20)
+        self.assertEqual(list(self.model), p(0, 19, 20, 3, 4, 5))
+        self.assertEqual(len(changed), 3)  # two are from before
+        assert_changed(1, 2, 0)
+        self.assertEqual(len(inserted), 1)  # from before
+        self.assertEqual(len(aboutinserted), 1)  # from before
+        self.assertEqual(len(removed), 1)  # from before
+        self.assertEqual(len(aboutremoved), 1)  # from before
+
+        # Insert without change
+        self.model[3:3] = p(21, 22)
+        self.assertEqual(list(self.model), p(0, 19, 20, 21, 22, 3, 4, 5))
+        self.assertEqual(len(changed), 3)  #from before
+        self.assertEqual(inserted[-1][1:], [3, 4])
+        self.assertEqual(aboutinserted[-1][1:], [3, 4])
+        self.assertEqual(len(removed), 1)  # from before
+        self.assertEqual(len(aboutremoved), 1)  # from before
+
+        # Remove without change
+        self.model[3:5] = []
+        self.assertEqual(list(self.model), p(0, 19, 20, 3, 4, 5))
+        self.assertEqual(len(changed), 3)  #from before
+        self.assertEqual(removed[-1][1:], [3, 4])
+        self.assertEqual(aboutremoved[-1][1:], [3, 4])
+        self.assertEqual(len(inserted), 2)  # from before
+        self.assertEqual(len(aboutinserted), 2)  # from before
+
+        # Remove all
+        self.model[:] = []
+        self.assertEqual(list(self.model), [])
+        self.assertEqual(len(changed), 3)  #from before
+        self.assertEqual(removed[-1][1:], [0, 5])
+        self.assertEqual(aboutremoved[-1][1:], [0, 5])
+        self.assertEqual(len(inserted), 2)  # from before
+        self.assertEqual(len(aboutinserted), 2)  # from before
+
+        # Add to empty
+        self.model[:] = p(0, 1, 2, 3)
+        self.assertEqual(list(self.model), p(0, 1, 2, 3))
+        self.assertEqual(len(changed), 3)  #from before
+        self.assertEqual(inserted[-1][1:], [0, 3])
+        self.assertEqual(inserted[-1][1:], [0, 3])
+        self.assertEqual(len(removed), 3)  # from before
+        self.assertEqual(len(aboutremoved), 3)  # from before
+
 
 class TestAbstractSortTableModel(unittest.TestCase):
     def test_sorting(self):
@@ -159,6 +252,40 @@ class TestAbstractSortTableModel(unittest.TestCase):
         self.assertSequenceEqual(model.mapToSourceRows(...).tolist(), [0, 2, 1])
         self.assertSequenceEqual(model.mapFromSourceRows(...).tolist(), [0, 2, 1])
 
+    def test_sorting_2d(self):
+        class Model(AbstractSortTableModel):
+            def rowCount(self):
+                return 3
+
+            def sortColumnData(self, _):
+                return np.array([[4, 6, 2],
+                                 [3, 3, 3],
+                                 [4, 6, 1]])
+        model = Model()
+        model.sort(0)
+        self.assertEqual(model.mapToSourceRows(...).tolist(), [1, 2, 0])
+
+    def test_setSortIndices(self):
+        model = AbstractSortTableModel()
+        model.rowCount = lambda: 5
+        spy_about = QSignalSpy(model.layoutAboutToBeChanged)
+        spy_changed = QSignalSpy(model.layoutChanged)
+
+        model.setSortIndices([4, 0, 1, 3, 2])
+        self.assertEqual(len(spy_about), 1)
+        self.assertEqual(len(spy_changed), 1)
+        self.assertEqual(model.mapFromSourceRows(...).tolist(), [1, 2, 4, 3, 0])
+        self.assertEqual(model.mapToSourceRows(...).tolist(), [4, 0, 1, 3, 2])
+
+        rows = [0, 1, 2, 3, 4]
+        model.setSortIndices(None)
+        self.assertEqual(len(spy_about), 2)
+        self.assertEqual(len(spy_changed), 2)
+        self.assertEqual(model.mapFromSourceRows(...), ...)
+        self.assertEqual(model.mapToSourceRows(...), ...)
+        self.assertEqual(model.mapFromSourceRows(rows), rows)
+        self.assertEqual(model.mapToSourceRows(rows), rows)
+
 
 # Tests test _is_index_valid and access model._other_data. The latter tests
 # implementation, but it would be cumbersome and less readable to test function
