@@ -9,7 +9,7 @@ from math import log10
 from typing import Optional, Union, List
 
 from AnyQt.QtWidgets import (
-    QWidget, QDialog, QVBoxLayout, QSizePolicy, QApplication, QStyle,
+    QWidget, QDialog, QVBoxLayout, QSizePolicy, QStyle,
     QShortcut, QSplitter, QSplitterHandle, QPushButton, QStatusBar,
     QProgressBar, QAction, QFrame, QStyleOption, QWIDGETSIZE_MAX,
     QHBoxLayout)
@@ -369,8 +369,9 @@ class OWBaseWidget(QDialog, OWComponent, Report, ProgressBarMixin,
                 warnings.warn(
                     "subclassing of widget classes is deprecated and will be "
                     "disabled in the future.\n"
-                    f"Extract code from {base.__name__} or explicitly open it.",
-                    RuntimeWarning)
+                    f"Extract code from {base.__name__} or explicitly open it "
+                    "by adding `openclass=True` to class definition.",
+                    RuntimeWarning, stacklevel=3)
                 # raise TypeError(f"class {base.__name__} cannot be subclassed")
 
     @classmethod
@@ -452,8 +453,8 @@ class OWBaseWidget(QDialog, OWComponent, Report, ProgressBarMixin,
                     x0, x1 = 6, w
                 else:
                     x0, x1 = w, 6
-                y = self.height() / 2
-                h = (w - 6) / 1.12
+                y = self.height() // 2
+                h = int((w - 6) / 1.12)
                 painter.setRenderHint(painter.Antialiasing)
                 painter.drawLines(
                     QLine(x0, y - h, x1, y),
@@ -889,7 +890,7 @@ class OWBaseWidget(QDialog, OWComponent, Report, ProgressBarMixin,
             geom.x() + diffx, geom.y(), geom.width() + diffw, geom.height()
         )
         # bound/move by available geometry
-        bounds = QApplication.desktop().availableGeometry(self)
+        bounds = self.screen().availableGeometry()
         bounds = bounds.adjusted(
             framemargins.left(), framemargins.top(),
             -framemargins.right(), -framemargins.bottom()
@@ -939,7 +940,7 @@ class OWBaseWidget(QDialog, OWComponent, Report, ProgressBarMixin,
 
         if restored and not self.windowState() & \
                 (Qt.WindowMaximized | Qt.WindowFullScreen):
-            space = QApplication.desktop().availableGeometry(self)
+            space = self.screen().availableGeometry()
             frame, geometry = self.frameGeometry(), self.geometry()
 
             # Fix the widget size to fit inside the available space
@@ -992,7 +993,7 @@ class OWBaseWidget(QDialog, OWComponent, Report, ProgressBarMixin,
         if self.want_main_area:
             width += self.__splitter.handleWidth()
             if self.want_control_area:
-                width += height * self.mainArea_width_height_ratio
+                width += int(height * self.mainArea_width_height_ratio)
             else:
                 return super().sizeHint()
         return QSize(width, height)
